<?php

namespace App\Command;

use App\Entity\Client;
use App\Repository\ClientRepository;
use League\Bundle\OAuth2ServerBundle\ValueObject\Grant as GrantVO;
use League\Bundle\OAuth2ServerBundle\ValueObject\RedirectUri as RedirectUriVO;
use League\Bundle\OAuth2ServerBundle\ValueObject\Scope as ScopeVO;
use Symfony\Component\Console\Attribute\AsCommand;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputArgument;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Console\Style\SymfonyStyle;

#[AsCommand(
    name: 'app:create-oauth-client',
    description: 'Creates a new OAuth client',
)]
class CreateOAuthClientCommand extends Command
{
    public function __construct(
        private ClientRepository $clientRepository
    ) {
        parent::__construct();
    }

    protected function configure(): void
    {
        $this
            ->addArgument('identifier', InputArgument::REQUIRED, 'Client identifier (max 32 chars)')
            ->addArgument('name', InputArgument::REQUIRED, 'Client name')
            ->addArgument('redirect-uri', InputArgument::REQUIRED, 'Redirect URI')
            ->addOption('grant-type', null, InputOption::VALUE_REQUIRED | InputOption::VALUE_IS_ARRAY, 'Grant type (authorization_code, client_credentials, implicit, password, refresh_token)', ['authorization_code'])
            ->addOption('scope', null, InputOption::VALUE_REQUIRED | InputOption::VALUE_IS_ARRAY, 'Scope', ['email'])
            ->addOption('secret', null, InputOption::VALUE_OPTIONAL, 'Client secret (for confidential clients)')
        ;
    }

    protected function execute(InputInterface $input, OutputInterface $output): int
    {
        $io = new SymfonyStyle($input, $output);
        $identifier = $input->getArgument('identifier');
        $name = $input->getArgument('name');
        $redirectUri = $input->getArgument('redirect-uri');
        $grantTypes = $input->getOption('grant-type');
        $scopes = $input->getOption('scope');
        $secret = $input->getOption('secret');

        if (strlen($identifier) > 32) {
            $io->error('Identifier must be 32 characters or less');
            return Command::FAILURE;
        }

        if ($this->clientRepository->find($identifier)) {
            $io->error(sprintf('Client with identifier "%s" already exists', $identifier));
            return Command::FAILURE;
        }

        $client = new Client($identifier, $secret, $name);
        
        // Create and spread the redirect URI
        $client->setRedirectUris(new RedirectUriVO($redirectUri));
        
        // Create the grant type objects
        $grantObjects = array_map(fn($grantType) => new GrantVO($grantType), $grantTypes);
        // Spread the grant objects to the setter
        $client->setGrants(...$grantObjects);
        
        // Create the scope objects
        $scopeObjects = array_map(fn($scope) => new ScopeVO($scope), $scopes);
        // Spread the scope objects to the setter
        $client->setScopes(...$scopeObjects);

        $this->clientRepository->save($client, true);

        $io->success(sprintf('Client "%s" created successfully', $identifier));
        $io->table(
            ['Property', 'Value'],
            [
                ['Identifier', $identifier],
                ['Secret', $secret ?: 'none (public client)'],
                ['Name', $name],
                ['Redirect URI', $redirectUri],
                ['Grant Types', implode(', ', $grantTypes)],
                ['Scopes', implode(', ', $scopes)],
            ]
        );

        return Command::SUCCESS;
    }
} 