<?php

namespace App\Entity;

use App\Repository\ClientRepository;
use Doctrine\ORM\Mapping as ORM;
use League\OAuth2\Server\Entities\ClientEntityInterface;
use League\Bundle\OAuth2ServerBundle\ValueObject\ClientMetadata;
use League\Bundle\OAuth2ServerBundle\ValueObject\Grant as GrantVO;
use League\Bundle\OAuth2ServerBundle\ValueObject\RedirectUri as RedirectUriVO;
use League\Bundle\OAuth2ServerBundle\ValueObject\Scope as ScopeVO;

#[ORM\Entity(repositoryClass: ClientRepository::class)]
class Client implements ClientEntityInterface
{
    #[ORM\Id]
    #[ORM\Column(length: 32)]
    private string $identifier;

    #[ORM\Column]
    private ?string $secret = null;

    #[ORM\Column]
    private string $name;

    #[ORM\Column]
    private bool $active = true;

    #[ORM\Column]
    private array $redirectUris = [];

    #[ORM\Column]
    private array $grants = [];

    #[ORM\Column]
    private array $scopes = [];

    #[ORM\Column(options: ["default" => false])]
    private bool $allowPlainTextPkce = false;

    public function __construct(string $identifier, ?string $secret, string $name)
    {
        $this->identifier = $identifier;
        $this->secret = $secret;
        $this->name = $name;
    }

    public function getIdentifier(): string
    {
        return $this->identifier;
    }

    public function getSecret(): ?string
    {
        return $this->secret;
    }

    public function getName(): string
    {
        return $this->name;
    }

    public function isActive(): bool
    {
        return $this->active;
    }

    public function setActive(bool $active): self
    {
        $this->active = $active;
        return $this;
    }

    public function getRedirectUri(): string|array
    {
        $uris = array_map(
            static fn (string $redirectUri): string => $redirectUri,
            $this->redirectUris
        );
        
        return $uris;
    }

    public function getGrants(): array
    {
        return array_map(
            static fn (string $grant): GrantVO => new GrantVO($grant),
            $this->grants
        );
    }

    public function setGrants(GrantVO ...$grants): self
    {
        $this->grants = array_map(
            static fn (GrantVO $grant): string => (string) $grant,
            $grants
        );
        return $this;
    }

    public function getScopes(): array
    {
        return array_map(
            static fn (string $scope): ScopeVO => new ScopeVO($scope),
            $this->scopes
        );
    }

    public function setScopes(ScopeVO ...$scopes): self
    {
        $this->scopes = array_map(
            static fn (ScopeVO $scope): string => (string) $scope,
            $scopes
        );
        return $this;
    }

    public function getMetadata(): ClientMetadata
    {
        return new ClientMetadata($this->name);
    }
    
    public function isConfidential(): bool
    {
        return $this->secret !== null;
    }
    
    public function isPlainTextPkceAllowed(): bool
    {
        return $this->allowPlainTextPkce;
    }
    
    public function setAllowPlainTextPkce(bool $allowPlainTextPkce): self
    {
        $this->allowPlainTextPkce = $allowPlainTextPkce;
        return $this;
    }
    
    public function __toString(): string
    {
        return $this->getIdentifier();
    }

    public function getRedirectUris(): array
    {
        return array_map(
            static fn (string $redirectUri): RedirectUriVO => new RedirectUriVO($redirectUri),
            $this->redirectUris
        );
    }

    public function setRedirectUris(RedirectUriVO ...$redirectUris): self
    {
        $this->redirectUris = array_map(
            static fn (RedirectUriVO $redirectUri): string => (string) $redirectUri,
            $redirectUris
        );
        return $this;
    }
} 